unit Px7Table;

(*

This is a TTable descendat with an added method to make use of the
new Pdox7 descending indices. With this type of index, you can
determine the sortorder for every single field in the index.

AddPx7Index works just like TTable.Addindex except for an additional
string parameter named DescFields:

procedure AddPx7Index(const Name,
                     Fields,
                     DescFields: string;
                     Options: TIndexOptions);

DescFields is only evaluated with ixDescending in Options, so else
you can supply an empty string. With ixDescending it holds the list
of fields that shall actually be sorted descending, i.e. for a full
descending index it equals the Fields parameter.

The following example creates an index that is ascending on the
first field and descending on the 2nd and 3rd:

AddPx7Index('AscDescMix',
           'Field1;Field2;Field3',
           'Field2;Field3',
            [ixDescending]);

*)

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, DB, DBTables, DBIProcs, DBITypes;

type
  TPx7Table = class(TTable)
  private
    procedure EncodePx7IndexDesc(var IndexDesc: IDXDesc;
      const Name, Fields, DescFields: string;
      Options: TIndexOptions);
    function IsPxTable: Boolean;
  protected
  public
    procedure AddPx7Index(const Name, Fields, DescFields: string;
      Options: TIndexOptions);
  published
  end;

procedure Register;

implementation

procedure TPx7Table.AddPx7Index(const Name, Fields,
                                DescFields: string;
                                Options: TIndexOptions);
var
  STableName: DBITBLNAME;
  IndexDesc: IDXDesc;
begin
  if not IsPxTable then
    AddIndex(Name, Fields, Options) {or raise an exception}
  else
  begin
    FieldDefs.Update;
    EncodePx7IndexDesc(IndexDesc, Name, Fields, DescFields, Options);
    if Active then
    begin
      CheckBrowseMode;
      CursorPosChanged;
      Check(DbiAddIndex(DBHandle, Handle, nil, nil, IndexDesc, nil));
    end else
    begin
      SetDBFlag(dbfTable, True);
      try
        Check(DbiAddIndex(DBHandle, nil, AnsiToNative(DBLocale, TableName,
          STableName, SizeOf(STableName) - 1), szParadox,
          IndexDesc, nil));
      finally
        SetDBFlag(dbfTable, False);
      end;
    end;
    DataEvent(dePropertyChange, 0);
  end;
end;

procedure TPx7Table.EncodePx7IndexDesc(var IndexDesc: IDXDesc;
  const Name, Fields, DescFields: string; Options: TIndexOptions);
var
  iPos, jPos: Integer;
begin
  FillChar(IndexDesc, SizeOf(IndexDesc), 0);
  with IndexDesc do
  begin
    AnsiToNative(DBLocale, Name, szName, SizeOf(szName) - 1);
    bPrimary := ixPrimary in Options;
    bUnique := ixUnique in Options;
    bDescending := ixDescending in Options;
    bCaseInsensitive := ixCaseInsensitive in Options;
    bMaintained := True;
    iPos := 1;
    while (iPos <= Length(Fields)) and (iFldsInKey < 16) do
    begin
      jPos := iPos;
      aiKeyFld[iFldsInKey] :=
        FieldDefs.Find(ExtractFieldName(Fields, iPos)).FieldNo;
      if bDescending
      and (pos(ExtractFieldName(Fields, jPos), DescFields)<>0) then
        iUnUsed[iFldsInKey] := 1;
      Inc(iFldsInKey);
    end;
  end;
end;

function TPx7Table.IsPxTable: Boolean;
begin
  Result := (TableType = ttParadox) or
    (CompareText(ExtractFileExt(TableName), '.DB') = 0);
end;

procedure Register;
begin
  RegisterComponents('DBAddOns', [TPx7Table]);
end;

end.
